/*******************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2004-2005 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without 
any warranty. CUSTOMER retains the final decision relative to the total design 
and functionality of the end product. FREESCALE neither guarantees nor will be 
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING, 
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR 
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE ORE ADVISE SUPPLIED TO AN PROJECT
BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE SERVICES. IN NO EVENT
SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF 
THIS AGREEMENT.

CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands or 
actions by anyone on account of any damage, or injury, whether commercial, 
contractual, or tortuous, rising directly or indirectly as a result of an advise
or assistance supplied CUSTOMER in connection with product, services or goods 
supplied under this Agreement.
********************************************************************************
* File              main.c
* Owner             r62780
* Version           1.1
* Date              Oct-10-2011
* Classification    General Business Information
* Brief             XEP100 external bus communication with MR2A16ACYS35
********************************************************************************
* Detailed Description:

*   - The example presents basic principle of communication with an external
      memory MR2A16ACYS35 256kx16bit 3.3-V Asynchronous Magnetoresistive RAM.
      The memory contains UDS and LDS signal and 
      it is suitable for Byte and Word (aligned or misaligned) access.
      External space assigned to \CS1 is used for the memory addressing.

*   - Tested on: EVB MC9S12XEP100 
*   - Source clock(Loop Controlled Pierce) OSCCLK = 16MHz, BUSCLK = 8MHz
      PLL is not used
*   - Reference to documentation: MC9S12XEP100RMV1 Rev. 1.19 12/2008

********************************************************************************
Revision History:
Version  Date         Author  Description of Changes
1.0      Nov-16-2010  R62780  Initial version
*******************************************************************************/

#include <hidef.h>      /* common defines and macros */
#include "derivative.h"      /* derivative-specific definitions */

/*******************************************************************************
* External objects
*******************************************************************************/

// none

/*******************************************************************************
* Constants and macros
*******************************************************************************/
// Macros for direct access to the external RAM. Address is seen from 
// the memory side

// 512kB SRAM - address space 0x00000~0x7FFFF
// byte access or alligned/misaligned word access
#define  WRITE_BYTE_B(a,d)  *((unsigned char *far)(a+0x200000UL))=d
#define  READ_BYTE_B(a)     *((unsigned char *far)(a+0x200000UL))
#define  WRITE_WORD_B(a,d)  *((unsigned int  *far)(a+0x200000UL))=d
#define  READ_WORD_B(a)     *((unsigned int  *far)(a+0x200000UL))

// 256kWord SRAM - address space 0x00000~0x3FFFF
// always alligned access to word
#define  WRITE_WORD_W(a,d)  *((unsigned int  *far)((a<<1)+0x200000UL))=d
#define  READ_WORD_W(a)     *((unsigned int  *far)((a<<1)+0x200000UL))

/*******************************************************************************
* Global variables
*******************************************************************************/

//..........................................
// for space definition look into prm file lines 68,69 and  198,199
//..........................................

#pragma DATA_SEG __GPAGE_SEG MY_EXSRAM0_CS1 //space for initialized variables 

static   unsigned char far arr_ub[10];      // array is cleared during autoinit   
static   unsigned int  far arr_uw[10];      // array is cleared during autoinit

volatile unsigned int far xxx0 = 0x1234;    // var is init. during autoinit

#pragma DATA_SEG DEFAULT 

//..........................................
// for space definition look into prm file lines 68,69 and  198,199
//..........................................
#pragma DATA_SEG __GPAGE_SEG MY_EXSRAM1_CS1 // space for uninitializes variables
                                            // variables will not be initialized
                                            // even it is done in their
                                            // definitions
volatile unsigned int far xxx1 = 0x789A;

#pragma DATA_SEG DEFAULT 

/*******************************************************************************
Function Name : main
Engineer      : r62780
Date          : Oct-10-2011
Parameters    : NONE 
Modifies      : NONE
Returns       : NONE
Notes         : Plays game with external memory
Issues        : NONE
*******************************************************************************/
void main(void) 
{
  unsigned long addr,idx;
  unsigned char ub;
  unsigned int  uw;
/*
  //====================================== 
  for bus initialization see start12.c file line 413
  also #include "derivative.h"  is included int start12.c
  Do not forget fulfill condition described in the start12.c file at line 40=>
   add -D__FAR_DATA into the compiler command line
  //========================================
*/

  //....................................
  xxx1 = 0x2345;                        // innitialize uninitialized variable
  //....................................

  
  for(;;) 
    {
      
      // fill entire SRAM byte by byte; seen from SRAM address space
      ub = 0;
      for(addr = 0; addr < 0x80000UL; addr++)  //512 kB
        {
          WRITE_BYTE_B(addr,ub++);             // write RAM address
        }


      // fill entire SRAM word by word; seen from SRAM address space
      // aligned adresses
      uw = 0xAAAA;
      for(addr = 0; addr < 0x80000UL; addr+=2) //256 kWord
        {
          WRITE_WORD_B(addr,uw++);             // write RAM address
        }


      // fill entire SRAM word by word; seen from SRAM address space
      // misaligned adresses
      uw = 0xBBBB;
      for(addr = 1; addr < 0x80000UL; addr+=2) //256 kWord
        {
          WRITE_WORD_B(addr,uw++);             // write RAM address
        }


      // fill entire SRAM word by word; seen from SRAM address space; 
      // aligned addresses
      uw = 0x1111;
      for(addr = 0; addr < 0x40000UL; addr++)  //256 kWord
        {
          WRITE_WORD_W(addr,uw++);             // write RAM address
        }
        
      // write array variables inside ExRAM
      ub = 0x22;
      for(idx = 0; idx < 10; idx++)
        {
          arr_ub[idx]=ub++;                    // fill array
        }

      uw = 0x3333;
      for(idx = 0; idx < 10; idx++)
        {
          arr_uw[idx]=uw++;                    // fill array
        }
    }
}
/*******************************************************************************
The Answer to the Ultimate Question of Life, the Universe, and Everything is: 42
(Douglas Adams - The Hitchhiker's Guide to the Galaxy)
*******************************************************************************/
